
/*
 * Esta función se encarga de manejar los mensajes recibidos del canal suscrito a través del broker MQTT
 *   topic - Tema de suscripción para mensaje.
 *   messege - Mensaje recibido en formato byte.
 *   length - Longitud del mensaje.
 */

int myMessageArrived(char* topic, byte* messege, unsigned int length)
{
  
  
  Serial.print("Mensaje recibido en tema: ");
  Serial.println(topic);
 
  Serial.print("Message:");

  char buffer[128];

  // Convierte el mensaje de bytes a int
  //Forma un string a partir de la carga útil
  memcpy(buffer, messege, length);
  buffer[length] = '\0';

  //Lo convierte en entero
  char *end = nullptr;
  long value = strtol(buffer, &end, 10);

  Serial.println(value);

  // Si un mensaje es recibido en el tema correspondiente (en el que se subscribe), se cambia el estado del LED según el valor de dicho mensaje.
  if (String(topic) == "channels/1232890/subscribe/fields/field1/FXTQY1CD8JH4CHFP") {

    //Si se recibe un 1 el LED se enciende y si se recibe un 0 se apaga.
    if(value == 1)
    {
      Serial.println("LED ON\n");
      digitalWrite(LED, HIGH);
    }
    else if(value == 0)
    {
      Serial.println("LED OFF\n");
      digitalWrite(LED, LOW);
    }
  
  }
}

/*
 * Esta función realiza la conexion con el cliente
 *   clientID - Identificador unico del cliente, en este caso se crea uno aleatoriamente con la funcion getID.
 *   clientName - Nombre del cliente que se ha establecido.
 *   MQTTAPIKey - Clave obtenida de ThingSpeak para realizar la conexion MQTT.
 */
 
void mqttConnect()
{  
    char clientID[ 9 ];
    
    // El bucle se repite hasta que se realice la conexión
    while ( !mqttClient.connected() )
    {

        getID(clientID,8);
       
        Serial.println( "Intentando conexion MQTT..." );

        
        if ( mqttClient.connect(clientID, clientName, MQTTAPIKey ))    
        {
            Serial.println( "Conectado con ClienID: " + String( clientID ) + "\t clientName"+ String( clientName ) + "\t MQTTAPIKey "+String(  MQTTAPIKey ) );
           
        }
        else
        {
            Serial.print( "La conexión ha fallado, estado = " );
            Serial.print( mqttClient.state() );
            Serial.println( "Reintentar en 5 segundos" );
            delay( 5000 );
        }
    }
}

/*
 * Esta funcion crea un identificador de cliente aleatorio, para que sea unico
 *   clientID - Matriz de caracteres para la salida
 *   idLength - Longitud clientID
 */

void getID(char clientID[], int idLength)
{
  static const char alphanum[] ="0123456789"
  "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
  "abcdefghijklmnopqrstuvwxyz";                        

    // Genera el identificador del cliente (client ID).
    for (int i = 0; i < idLength ; i++) 
    {
        clientID[ i ] = alphanum[ random( 51 ) ];
    }
    clientID[ idLength ] = '\0';
    
}



/**
 * Funcion que realiza la subscripcion a campos de un canal.
 *   subChannelID - Canal al que se subscribe.
 *   field - Campo al que se subscribe. Valor 0 significa que se subscibe a todos los campos del canal.
 *   readKey - Read API key del canal de subscripcion.
 *   
 */
  
int mqttSubscribe(unsigned long subChannelID, int field, char* readKey)
{
    String myTopic;
    
    if (field==0)
    {
      //Tema para subscribirse a todos los campos del canal
        myTopic="channels/"+String( subChannelID )+"/subscribe/json/"+String( readKey );
    }
    else
    {
      //Tema para subscribirse a un campo del canal
        myTopic="channels/"+String( subChannelID )+"/subscribe/fields/field"+String( field )+"/"+String( readKey );
    }

    
    Serial.println( "Subscribiendose a " +myTopic );

    return mqttClient.subscribe( myTopic.c_str() ,0 );
    
}

/*
 * Funcion que publica datos a un canal
 *   pubChannelID - Canal en el que se publica.
 *   pubWriteAPIKey - Write API key del canal al que publica.
 *   dataSend[] - Array con los datos que se van a publicar de temperatura y humedad
 *   
 */

void mqttPublish(unsigned long pubChannelID, char* pubWriteAPIKey, float dataSend[])
{
  
  float temp;
  float hum;
  
  temp = dataSend[0];
  hum = dataSend[1];

 //Extraemos en una variable la informacion útil que se va a enviar
  String payload="field1=";
  payload+=temp;
  payload+=" &field2=";
  payload+=hum;
  payload+=" &status=MQTTPUBLISH";
  
  
  // Tema en el que publicar.
  String topicString ="channels/" + String( pubChannelID ) + "/publish/"+String( pubWriteAPIKey );
  // Publicacion de los datos en el tema correspondiente
  mqttClient.publish( topicString.c_str(), payload.c_str() );
  Serial.println( "\nPublicado en el canal " + String( pubChannelID ) + "\n");  

}
